@@ -16,6 +16,7 @@ from .utils import (
     get_histogram_axes_title,
     get_plottable_protocol_bins,
     hist_object_handler,
+    isLight,
     process_histogram_parts,
 )
 
@@ -24,7 +25,7 @@ if TYPE_CHECKING:
 
 StairsArtists = namedtuple("StairsArtists", "stairs errorbar legend_artist")
 ErrorBarArtists = namedtuple("ErrorBarArtists", "errorbar")
-ColormeshArtists = namedtuple("ColormeshArtists", "pcolormesh cbar")
+ColormeshArtists = namedtuple("ColormeshArtists", "pcolormesh cbar text")
 
 
 Hist1DArtists = Union[StairsArtists, ErrorBarArtists]
@@ -137,7 +138,7 @@ def histplot(
             Label for legend entry.
         edges : bool, default: True, optional
             Specifies whether to draw first and last edges of the histogram
-        binticks : bool, optional
+        binticks : bool, default: False, optional
             Attempts to draw x-axis ticks coinciding with bin boundaries if feasible.
         ax : matplotlib.axes.Axes, optional
             Axes object (if None, last one is fetched or one is created)
@@ -164,7 +165,7 @@ def histplot(
     assert histtype in _allowed_histtype, _err_message
 
     hists = list(process_histogram_parts(H, bins))
-    final_bins = get_plottable_protocol_bins(hists[0].axes[0])
+    final_bins, xtick_labels = get_plottable_protocol_bins(hists[0].axes[0])
 
     # TODO: use hists everywhere
     h = np.stack([h.values().astype(float) for h in hists])
@@ -400,9 +401,13 @@ def histplot(
     # Add sticky edges for autoscale
     _artist.sticky_edges.y.append(0)
 
-    if binticks:
-        _slice = int(round(float(len(final_bins)) / len(ax.get_xticks()))) + 1
-        ax.set_xticks(final_bins[::_slice])
+    if xtick_labels is None:
+        if binticks:
+            _slice = int(round(float(len(final_bins)) / len(ax.get_xticks()))) + 1
+            ax.set_xticks(final_bins[::_slice])
+    else:
+        ax.set_xticks(_bin_centers)
+        ax.set_xticklabels(xtick_labels)
 
     if x_axes_label:
         ax.set_xlabel(x_axes_label)
@@ -435,7 +440,7 @@ def hist2dplot(
 
         - `np.histogram` tuple
         - `boost_histogram` histogram object
-        - raw histogram values, provided `bins` is specified.
+        - raw histogram values as list of list or 2d-array
 
     xbins : 1D array-like, optional, default None
         Histogram bins along x axis, if not part of ``H``.
@@ -486,8 +491,10 @@ def hist2dplot(
 
     # TODO: use Histogram everywhere
     H = hist.values()
-    xbins = get_plottable_protocol_bins(hist.axes[0])
-    ybins = get_plottable_protocol_bins(hist.axes[1])
+    xbins, xtick_labels = get_plottable_protocol_bins(hist.axes[0])
+    ybins, ytick_labels = get_plottable_protocol_bins(hist.axes[1])
+    xbin_centers = xbins[1:] - np.diff(xbins) / float(2)
+    ybin_centers = ybins[1:] - np.diff(ybins) / float(2)
 
     x_axes_label = get_histogram_axes_title(hist.axes[0])
     y_axes_label = get_histogram_axes_title(hist.axes[1])
@@ -512,10 +519,18 @@ def hist2dplot(
     ax.set_xlim(xbins[0], xbins[-1])
     ax.set_ylim(ybins[0], ybins[-1])
 
-    if len(ax.get_xticks()) > len(xbins) * 0.7:
-        ax.set_xticks(xbins)
-    if len(ax.get_yticks()) > len(ybins) * 0.7:
-        ax.set_yticks(ybins)
+    if xtick_labels is None:  # Ordered axis
+        if len(ax.get_xticks()) > len(xbins) * 0.7:
+            ax.set_xticks(xbins)
+    else:  # Categorical axis
+        ax.set_xticks(xbin_centers)
+        ax.set_xticklabels(xtick_labels)
+    if ytick_labels is None:
+        if len(ax.get_yticks()) > len(ybins) * 0.7:
+            ax.set_yticks(ybins)
+    else:  # Categorical axis
+        ax.set_yticks(ybin_centers)
+        ax.set_yticklabels(ytick_labels)
 
     if cbar:
         cax = append_axes(
@@ -527,23 +542,31 @@ def hist2dplot(
 
     plt.sca(ax)
 
+    text_artists = []
     if labels is not None:
         if labels is True:
             _labels = H
-        elif H.shape == labels.shape:
-            _labels = labels
+        elif H.shape == labels.T.shape:
+            _labels = labels.T
         elif labels is not False:
             raise ValueError(
-                "Labels not understood, either specify a bool or a" "Histlike array "
+                "Labels not understood, either specify a bool or a Hist-like array"
             )
-        _xbin_centers = xbins[1:] - np.diff(xbins) / float(2)
-        _ybin_centers = ybins[1:] - np.diff(ybins) / float(2)
-        for ix, xc in enumerate(_xbin_centers):
-            for iy, yc in enumerate(_ybin_centers):
-                color = "black" if pc.norm(H[iy, ix]) > 0.5 else "lightgrey"
-                ax.text(xc, yc, _labels[iy, ix], ha="center", va="center", color=color)
 
-    return ColormeshArtists(pc, cb_obj)
+        for ix, xc in enumerate(xbin_centers):
+            for iy, yc in enumerate(ybin_centers):
+                color = (
+                    "black"
+                    if isLight(pc.cmap(pc.norm(H[iy, ix]))[:-1])
+                    else "lightgrey"
+                )
+                text_artists.append(
+                    ax.text(
+                        xc, yc, _labels[iy, ix], ha="center", va="center", color=color
+                    )
+                )
+
+    return ColormeshArtists(pc, cb_obj, text_artists)
 
 
 #############################################
@@ -693,8 +716,6 @@ def mpl_magic(ax=None, info=True):
 
 ########################################
 # Figure/axes helpers
-
-
 def rescale_to_axessize(ax, w, h):
     """
     Adjust figure size to axes size in inches
@@ -833,8 +854,6 @@ def append_axes(ax, size=0.1, pad=0.1, position="right", extend=False):
 
 ####################
 # Legend Helpers
-
-
 def hist_legend(ax=None, **kwargs):
     from matplotlib.lines import Line2D
 
